import numpy as np
import sys
import xml.etree.ElementTree as ET
import time as clock_time

# Path to the place where the tool is installed
tool_path="/home/bsc32/bsc32402/local/magiccut"
# Path to the cutter template
template_path = tool_path + "/templates/cutter_template.xml"

# Read trace name
try:
   trace = sys.argv[1]
except IndexError:
   print("Necessary argument: trace with functions")
   exit(1)

print("Starting the magic")
start = clock_time.time()

# Read trace and load function calls
process, time, value = np.genfromtxt(trace, dtype='int', unpack='True')

print("for opening the trace: ")
end = clock_time.time()
print(end - start)

start = clock_time.time()

# Find set of different routines
func_id, call_counts = np.unique(value, return_counts=True)
print("for Find set of different routines: ", func_id.shape[0])
end = clock_time.time()
print(end - start)

# Find number of time-steps multiplied for the number of cores, idle and not
start_4ts_count = clock_time.time()

counts = np.bincount(call_counts)
time_steps =  np.argmax(counts)

end_4ts_count  = clock_time.time()

start = clock_time.time()

# Find functions that are called once per time-step
functions_called_once_per_step = func_id[call_counts == time_steps]
np.savetxt('func_one_per_ts.txt',functions_called_once_per_step, fmt='%d',)
print("for finding functions that are called once per time-step: ", functions_called_once_per_step.shape[0])
end = clock_time.time()
print(end - start)

start = clock_time.time()

# Find which routine is called in first place
first_routine = functions_called_once_per_step[0]
print("for finding  which routine is called in first place: ", first_routine)
end = clock_time.time()
print(end - start)

# Find number of nemo_proc_number
nemo_proc = process[value == first_routine]
nemo_proc = np.unique(nemo_proc)
nemo_proc_number = nemo_proc.shape[0]

nemo_proc_min=min(nemo_proc)
nemo_proc_max=max(nemo_proc)

time_steps = time[(value == first_routine) & (process == nemo_proc[0])].shape[0]
print("for finding number of time-steps: ", time_steps, "and the actual n_proc", nemo_proc_number)
print(end_4ts_count  - start_4ts_count )


start = clock_time.time()

# Find the index of the fastest step
time_step = np.ones([nemo_proc_number , time_steps])

ts_min_index = np.zeros(nemo_proc_number, dtype = 'int')
ts_max_index = np.zeros(nemo_proc_number, dtype = 'int')

ts_time = np.zeros([nemo_proc_number, time_steps])

for proc in nemo_proc:
    index = proc - nemo_proc[0]
    ts_time[ index ] = time[ (process == proc)  & (value == first_routine)]

    # compute the duration of each ts
    ts_duration = np.diff(ts_time[ index ])

    #find the index of the minimum ts for each proc
    ts_min_index[index] = np.argmin(ts_duration)

    #find the index of the maximum ts for each proc
    ts_max_index[index] = np.argmax(ts_duration[1:-1])

#evaluate the most common index for best ts
counts = np.bincount(ts_min_index)
best_ts_index =  np.argmax(counts)

counts = np.bincount(ts_max_index)
worst_ts_index = np.argmax(counts)

print("for finding the index of the slowest / fastest step: ", worst_ts_index, "/", best_ts_index)
end = clock_time.time()
print(end - start)

start = clock_time.time()

# Find the start and the end of the best step
best_ts_start = min(ts_time[:,best_ts_index-1])
best_ts_end = max(ts_time[:,best_ts_index])

# Find the start and the end of the best step
worst_ts_start = min(ts_time[:,worst_ts_index-1])
worst_ts_end   = max(ts_time[:,worst_ts_index])

print("for finding the worst / best time step's duration: ", best_ts_end -best_ts_start, worst_ts_end- worst_ts_start)
end = clock_time.time()
print(end - start)


# Open the xml template
tree = ET.parse(template_path)

cutter= tree.find('cutter') 
for tasks in tree.iter('tasks'):
  tasks.text=str(nemo_proc_min) + "-" + str(nemo_proc_max)
for minimum_time in tree.iter('minimum_time'):
  minimum_time.text=str(best_ts_start - 1000)
for maximum_time in tree.iter('maximum_time'):
  maximum_time.text=str(best_ts_end + 1000)

# Create paramedir cutter file
tree.write('best_time_cutter.xml')
for minimum_time in tree.iter('minimum_time'):
  minimum_time.text=str(worst_ts_start - 1000)
for maximum_time in tree.iter('maximum_time'):
  maximum_time.text=str(worst_ts_end + 1000)

# Create paramedir cutter file
tree.write('worst_time_cutter.xml')

