#!/bin/bash
###############################################################################
#                             SBATCH HEADER TEMPLATE
###############################################################################
#SBATCH --ntasks 48
#SBATCH -J magic-cut
#SBATCH --output EXP_FOLDER/magic_log_%j.out
#SBATCH --error  EXP_FOLDER/magic_log_%j.err
#SBATCH --time 00:15:00
#SBATCH --qos=debug

set -xv
source param.cfg

trace_files=`ls *.prv`

if  [[ -f ${trace_files} ]]
then
  tool_folder=/home/bsc32/bsc32402/local/magiccut
  module load intel/2017.4 impi/2017.4 mkl/2017.4 gcc/7.2.0 BSCTOOLS
  file_pcf=`basename ${trace_files} .prv`".pcf"
else
  echo "No trace file "${trace_files}" file found"
  exit 1
fi

func_check=`grep -m 1 60000019 ${file_pcf} | wc -l`
if [[ ${func_check} -gt 0 ]]
then
  name=${trace_files//.exe}
  name=${name//.prv}
# ---------------- old way ----- slow way --
#  functions_file=${tool_folder}/templates/filter_only_keep_functions.xml
#  paramedir -f ${functions_file} ./${trace_files} -o ./${name}_functions.prv
#  python ${tool_folder}/bin/find_time_to_cut.py ./${name}_functions.prv
## ---------------- new way ----- fast way --
  appo_file=my_file_prova.txt
  cat /dev/null > ${appo_file}
  grep  '60000019'  ${trace_files}  |\
  grep -v ':0:'  |\
  awk -F : '{print $2, $6, $8}' > ${appo_file}

  python ${tool_folder}/bin/find_time_to_cut.py ./${appo_file}
  rm ${appo_file}
# --------------- end of life choices --
 echo "start paramedir cutter"
 time paramedir -c best_time_cutter.xml ./${trace_files} -o ./${name}.best_cut.prv
 time paramedir -c worst_time_cutter.xml ./${trace_files} -o ./${name}.worst_cut.prv
else
  echo "Functions must be present in the trace for the script to work"
  exit
fi
######################### WIP : PRODUCING THE LIST OF BEST F #######
pcf_files=`basename $trace_file .prv`.pcf
initial_line=`grep -rin 60000019 $pcf_files | awk -F ':' '{print $1}'`
final_line=`sed -n ''${initial_line}', $p' $pcf_files | grep -m 1 -n  'EVENT_TYPE'| awk -F ':' '{print $1}'`
if [[ ${final_line} -gt `wc -l < func_one_per_ts.txt` ]]
then 
  echo "producing an optimal list of function to be traced"
  final_line=$((initial_line + final_line))
  sed -n ''$((initial_line+3))','$((final_line-2))'p' $pcf_files  | awk '$1 > 0 {print $1, $2}' | grep -w  -f func_one_per_ts.txt | awk '{print $2}' > list_of_best_func.txt
fi
rm func_one_per_ts.txt

